@@ -171,16 +171,8 @@ void  bacon_video_widget_set_logo_mode		 (BaconVideoWidget *bvw,
 						  gboolean logo_mode);
 gboolean bacon_video_widget_get_logo_mode	 (BaconVideoWidget *bvw);
 
-void bacon_video_widget_set_fullscreen		 (BaconVideoWidget *bvw,
-						  gboolean fullscreen);
-
 void bacon_video_widget_set_show_cursor          (BaconVideoWidget *bvw,
 						  gboolean show_cursor);
-gboolean bacon_video_widget_get_show_cursor      (BaconVideoWidget *bvw);
-
-gboolean bacon_video_widget_get_auto_resize	 (BaconVideoWidget *bvw);
-void bacon_video_widget_set_auto_resize		 (BaconVideoWidget *bvw,
-						  gboolean auto_resize);
 
 void bacon_video_widget_set_subtitle_font	 (BaconVideoWidget *bvw,
 						  const char *font);
@@ -355,9 +347,6 @@ void bacon_video_widget_set_aspect_ratio         (BaconVideoWidget *bvw,
 BvwAspectRatio bacon_video_widget_get_aspect_ratio
 						 (BaconVideoWidget *bvw);
 
-void bacon_video_widget_set_scale_ratio          (BaconVideoWidget *bvw,
-						  float ratio);
-
 void bacon_video_widget_set_zoom		 (BaconVideoWidget *bvw,
 						  BvwZoomMode       mode);
 BvwZoomMode bacon_video_widget_get_zoom		 (BaconVideoWidget *bvw);
@@ -423,11 +412,13 @@ GList *bacon_video_widget_get_languages          (BaconVideoWidget *bvw);
 int bacon_video_widget_get_language              (BaconVideoWidget *bvw);
 void bacon_video_widget_set_language             (BaconVideoWidget *bvw,
 		                                  int language);
+void bacon_video_widget_set_next_language        (BaconVideoWidget *bvw);
 
 GList *bacon_video_widget_get_subtitles          (BaconVideoWidget *bvw);
 int bacon_video_widget_get_subtitle              (BaconVideoWidget *bvw);
 void bacon_video_widget_set_subtitle             (BaconVideoWidget *bvw,
 		                                  int subtitle);
+void bacon_video_widget_set_next_subtitle        (BaconVideoWidget *bvw);
 
 gboolean bacon_video_widget_has_next_track	 (BaconVideoWidget *bvw);
 gboolean bacon_video_widget_has_previous_track	 (BaconVideoWidget *bvw);
@@ -464,8 +455,13 @@ void bacon_video_widget_set_audio_output_type    (BaconVideoWidget *bvw,
 						  BvwAudioOutputType type);
 
 /* OSD */
-void bacon_video_widget_popup_osd                 (BaconVideoWidget *bvw,
-						   const char       *icon_name);
+void bacon_video_widget_show_popup                (BaconVideoWidget *bvw);
+void bacon_video_widget_mark_popup_busy           (BaconVideoWidget *bvw,
+						   const char       *reason);
+void bacon_video_widget_unmark_popup_busy         (BaconVideoWidget *bvw,
+						   const char       *reason);
+
+GObject * bacon_video_widget_get_controls_object  (BaconVideoWidget *bvw);
 
 G_END_DECLS
 
