@@ -34,6 +34,7 @@
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
 #include <grilo.h>
+#include <pls/grl-pls.h>
 
 #include <totem-interface.h>
 #include <totem-dirs.h>
@@ -54,14 +55,23 @@
 #define PAGE_SIZE             50
 #define SCROLL_GET_MORE_LIMIT 0.8
 
+/* casts are to shut gcc up */
+static const GtkTargetEntry target_table[] = {
+	{ (gchar*) "text/uri-list", 0, 0 },
+	{ (gchar*) "_NETSCAPE_URL", 0, 1 }
+};
+
 struct _TotemGriloPrivate {
 	Totem *totem;
 	GtkWindow *main_window;
 
 	GrlSource *local_metadata_src;
 	GrlSource *metadata_store_src;
+	GrlSource *bookmarks_src;
 	gboolean fs_plugin_configured;
 
+	TotemGriloPage current_page;
+
 	/* Current media selected in results*/
 	GrlMedia *selected_media;
 
@@ -80,11 +90,15 @@ struct _TotemGriloPrivate {
 	GMenuModel *selectmenu;
 	GSimpleAction *select_all_action;
 	GSimpleAction *select_none_action;
+
+	/* Source switcher */
 	GtkWidget *switcher;
-	GtkWidget *recent, *channels;
+	GtkWidget *recent, *channels, *search_hidden_button;
+	char *last_page;
 
 	/* Browser widgets */
 	GtkWidget *browser;
+	guint dnd_handler_id;
 	GtkTreeModel *recent_model;
 	GtkTreeModel *recent_sort_model;
 	GtkTreeModel *browser_model;
@@ -107,7 +121,8 @@ enum {
 	PROP_0,
 	PROP_TOTEM,
 	PROP_HEADER,
-	PROP_SHOW_BACK_BUTTON
+	PROP_SHOW_BACK_BUTTON,
+	PROP_CURRENT_PAGE
 };
 
 G_DEFINE_TYPE_WITH_CODE (TotemGrilo, totem_grilo, GTK_TYPE_BOX,
@@ -154,6 +169,87 @@ enum {
 	CAN_REMOVE_TRUE        = 1
 };
 
+static gboolean
+strv_has_prefix (const char **strv,
+		 const char  *str)
+{
+	const char **s = strv;
+
+	while (*s) {
+		if (g_str_has_prefix (str, *s))
+			return TRUE;
+		s++;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+source_is_blacklisted (GrlSource *source)
+{
+	const char *id;
+	const char const *sources[] = {
+		"grl-shoutcast",
+		"grl-flickr",
+		"grl-podcasts",
+		"grl-dmap",
+		NULL
+	};
+
+	id = grl_source_get_id (source);
+	g_assert (id);
+
+	return strv_has_prefix (sources, id);
+}
+
+static gboolean
+source_is_browse_blacklisted (GrlSource *source)
+{
+	const char *id;
+	const char const *sources[] = {
+		/* https://bugzilla.gnome.org/show_bug.cgi?id=722422 */
+		"grl-youtube",
+		NULL
+	};
+
+	id = grl_source_get_id (source);
+	g_assert (id);
+
+	return strv_has_prefix (sources, id);
+}
+
+static gboolean
+source_is_search_blacklisted (GrlSource *source)
+{
+	const char *id;
+	const char const *sources[] = {
+		"grl-metadata-store",
+		NULL
+	};
+
+	id = grl_source_get_id (source);
+	g_assert (id);
+
+	return strv_has_prefix (sources, id);
+}
+
+static gboolean
+source_is_recent (GrlSource *source)
+{
+	const char *id;
+	const char const *sources[] = {
+		"grl-tracker-source",
+		"grl-optical-media",
+		"grl-bookmarks",
+		NULL
+	};
+
+	id = grl_source_get_id (source);
+	g_assert (id);
+
+	return strv_has_prefix (sources, id);
+}
+
 static gchar *
 get_secondary_text (GrlMedia *media)
 {
@@ -218,10 +314,10 @@ can_remove (GrlSource *source,
 	char *scheme;
 	int ret;
 
-	if (!(grl_source_supported_operations (source) & GRL_OP_REMOVE))
-		return CAN_REMOVE_UNSUPPORTED;
+	if (g_strcmp0 (grl_source_get_id (source), "grl-bookmarks") == 0)
+		return CAN_REMOVE_TRUE;
 	if (!media)
-		return CAN_REMOVE_FALSE;
+		goto fallback;
 	if (GRL_IS_MEDIA_BOX (media))
 		return CAN_REMOVE_FALSE;
 	url = grl_media_get_url (media);
@@ -232,7 +328,14 @@ can_remove (GrlSource *source,
 	ret = g_str_equal (scheme, "file") ? CAN_REMOVE_TRUE : CAN_REMOVE_FALSE;
 	g_free (scheme);
 
-	return ret;
+	if (ret == CAN_REMOVE_TRUE)
+		return CAN_REMOVE_TRUE;
+
+fallback:
+	if (!(grl_source_supported_operations (source) & GRL_OP_REMOVE))
+		return CAN_REMOVE_UNSUPPORTED;
+
+	return CAN_REMOVE_FALSE;
 }
 
 static void
@@ -301,10 +404,10 @@ out:
 }
 
 static void
-set_thumbnail_async (TotemGrilo *self,
-		     GrlMedia         *media,
-		     GtkTreeModel     *model,
-		     GtkTreePath      *path)
+set_thumbnail_async (TotemGrilo   *self,
+		     GrlMedia     *media,
+		     GtkTreeModel *model,
+		     GtkTreePath  *path)
 {
 	SetThumbnailData *thumb_data;
 
@@ -320,10 +423,10 @@ set_thumbnail_async (TotemGrilo *self,
 }
 
 static void
-set_thumbnail_source_async (TotemGrilo *self,
-			    GrlSource        *source,
-			    GtkTreeModel     *model,
-			    GtkTreePath      *path)
+set_thumbnail_source_async (TotemGrilo   *self,
+			    GrlSource    *source,
+			    GtkTreeModel *model,
+			    GtkTreePath  *path)
 {
 	SetThumbnailData *thumb_data;
 
@@ -447,15 +550,29 @@ update_media (GtkTreeStore *model,
 
 static void
 add_local_metadata (TotemGrilo *self,
-		    GrlMedia         *media)
+		    GrlSource  *source,
+		    GrlMedia   *media)
 {
 	GrlOperationOptions *options;
 
 	/* This is very slow and sync, so don't run it
 	 * for non-local media */
-	if (!g_str_equal (grl_media_get_source (media), "grl-tracker-source"))
+	if (!source_is_recent (source))
 		return;
 
+	/* Avoid trying to get metadata for web radios */
+	if (source == self->priv->bookmarks_src) {
+		char *scheme;
+
+		scheme = g_uri_parse_scheme (grl_media_get_url (media));
+		if (g_strcmp0 (scheme, "http") == 0 ||
+		    g_strcmp0 (scheme, "https") == 0) {
+			g_free (scheme);
+			return;
+		}
+		g_free (scheme);
+	}
+
 	options = grl_operation_options_new (NULL);
 	grl_operation_options_set_flags (options, GRL_RESOLVE_FULL);
 	grl_source_resolve_sync (self->priv->local_metadata_src,
@@ -482,6 +599,8 @@ get_source_priority (GrlSource *source)
 	id = grl_source_get_id (source);
 	if (g_str_equal (id, "grl-optical-media"))
 		return 100;
+	if (g_str_equal (id, "grl-bookmarks"))
+		return 75;
 	if (g_str_equal (id, "grl-tracker-source"))
 		return 50;
 	if (g_str_has_prefix (id, "grl-upnp-") ||
@@ -524,11 +643,11 @@ add_media_to_model (GtkTreeStore *model,
 }
 
 static void
-browse_cb (GrlSource *source,
-           guint browse_id,
-           GrlMedia *media,
-           guint remaining,
-           gpointer user_data,
+browse_cb (GrlSource    *source,
+           guint         browse_id,
+           GrlMedia     *media,
+           guint         remaining,
+           gpointer      user_data,
            const GError *error)
 {
 	BrowseUserData *bud;
@@ -571,7 +690,7 @@ browse_cb (GrlSource *source,
 			g_assert_not_reached ();
 		}
 
-		add_local_metadata (self, media);
+		add_local_metadata (self, source, media);
 		add_media_to_model (GTK_TREE_STORE (bud->model),
 				    bud->ref_parent ? &parent : NULL,
 				    source, media);
@@ -590,12 +709,12 @@ browse_cb (GrlSource *source,
 }
 
 static void
-browse (TotemGrilo *self,
-	GtkTreeModel     *model,
-        GtkTreePath      *path,
-        GrlSource        *source,
-        GrlMedia         *container,
-        gint              page)
+browse (TotemGrilo   *self,
+	GtkTreeModel *model,
+        GtkTreePath  *path,
+        GrlSource    *source,
+        GrlMedia     *container,
+        gint          page)
 {
 	BrowseUserData *bud;
 	GrlOperationOptions *default_options;
@@ -634,9 +753,9 @@ browse (TotemGrilo *self,
 
 static void
 play (TotemGrilo *self,
-      GrlSource *source,
-      GrlMedia *media,
-      gboolean resolve_url)
+      GrlSource  *source,
+      GrlMedia   *media,
+      gboolean    resolve_url)
 {
 	const gchar *url;
 	char *title;
@@ -660,11 +779,11 @@ play (TotemGrilo *self,
 }
 
 static void
-search_cb (GrlSource *source,
-           guint search_id,
-           GrlMedia *media,
-           guint remaining,
-           gpointer user_data,
+search_cb (GrlSource    *source,
+           guint         search_id,
+           GrlMedia     *media,
+           guint         remaining,
+           gpointer      user_data,
            const GError *error)
 {
 	GtkWindow *window;
@@ -689,7 +808,7 @@ search_cb (GrlSource *source,
 			g_assert_not_reached ();
 		}
 
-		add_local_metadata (self, media);
+		add_local_metadata (self, source, media);
 		add_media_to_model (GTK_TREE_STORE (self->priv->search_results_model),
 				    NULL, source, media);
 
@@ -763,7 +882,9 @@ search_more (TotemGrilo *self)
 }
 
 static void
-search (TotemGrilo *self, GrlSource *source, const gchar *text)
+search (TotemGrilo  *self,
+	GrlSource   *source,
+	const gchar *text)
 {
 	gtk_tree_store_clear (GTK_TREE_STORE (self->priv->search_results_model));
 //	g_hash_table_remove_all (self->priv->cache_thumbnails);
@@ -779,7 +900,8 @@ search (TotemGrilo *self, GrlSource *source, const gchar *text)
 }
 
 static void
-search_entry_activate_cb (GtkEntry *entry, TotemGrilo *self)
+search_entry_activate_cb (GtkEntry   *entry,
+			  TotemGrilo *self)
 {
 	GrlRegistry *registry;
 	const char *id;
@@ -840,7 +962,7 @@ set_browser_filter_model_for_path (TotemGrilo    *self,
 static void
 browser_activated_cb (GdMainView  *view,
                       GtkTreePath *path,
-                      gpointer user_data)
+                      gpointer     user_data)
 {
 	guint remaining;
 	gint page;
@@ -890,8 +1012,8 @@ free_data:
 }
 
 static void
-search_entry_source_changed_cb (GObject          *object,
-                                GParamSpec       *pspec,
+search_entry_source_changed_cb (GObject    *object,
+                                GParamSpec *pspec,
                                 TotemGrilo *self)
 {
 	/* FIXME: Do we actually want to do that? */
@@ -905,7 +1027,7 @@ search_entry_source_changed_cb (GObject          *object,
 static void
 search_activated_cb (GdMainView  *view,
                      GtkTreePath *path,
-                     gpointer user_data)
+                     gpointer     user_data)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -929,7 +1051,7 @@ static void
 item_activated_cb (GdMainView  *view,
 		   const char  *id,
 		   GtkTreePath *path,
-		   gpointer user_data)
+		   gpointer     user_data)
 {
 	TotemGrilo *self = TOTEM_GRILO (user_data);
 	GtkTreeModel *model;
@@ -944,73 +1066,6 @@ item_activated_cb (GdMainView  *view,
 	}
 }
 
-static gboolean
-strv_has_prefix (const char **strv,
-		 const char  *str)
-{
-	const char **s = strv;
-
-	while (*s) {
-		if (g_str_has_prefix (str, *s))
-			return TRUE;
-		s++;
-	}
-
-	return FALSE;
-}
-
-static gboolean
-source_is_blacklisted (GrlSource *source)
-{
-	const char *id;
-	const char const *sources[] = {
-		"grl-bookmarks",
-		"grl-shoutcast",
-		"grl-flickr",
-		"grl-podcasts",
-		"grl-dmap",
-		NULL
-	};
-
-	id = grl_source_get_id (source);
-	g_assert (id);
-
-	return strv_has_prefix (sources, id);
-}
-
-static gboolean
-source_is_browse_blacklisted (GrlSource *source)
-{
-	const char *id;
-	const char const *sources[] = {
-		/* https://bugzilla.gnome.org/show_bug.cgi?id=722422 */
-		"grl-youtube",
-		NULL
-	};
-
-	id = grl_source_get_id (source);
-	g_assert (id);
-
-	return strv_has_prefix (sources, id);
-}
-
-static gboolean
-source_is_search_blacklisted (GrlSource *source)
-{
-	const char *id;
-	const char const *sources[] = {
-		"grl-metadata-store",
-		NULL
-	};
-
-	id = grl_source_get_id (source);
-	g_assert (id);
-
-	return strv_has_prefix (sources, id);
-}
-
-
-
 static gboolean
 find_media_cb (GtkTreeModel  *model,
 	       GtkTreePath   *path,
@@ -1050,23 +1105,18 @@ find_media (GtkTreeModel *model,
 
 static GtkTreeModel *
 get_tree_model_for_source (TotemGrilo *self,
-			   GrlSource        *source)
+			   GrlSource  *source)
 {
-	const char *id;
-
-	id = grl_source_get_id (source);
-	if (g_str_equal (id, "grl-tracker-source") ||
-	    g_str_equal (id, "grl-optical-media")) {
+	if (source_is_recent (source))
 		return self->priv->recent_model;
-	}
 
 	return self->priv->browser_model;
 }
 
 static void
 content_changed (TotemGrilo   *self,
-		 GrlSource          *source,
-		 GPtrArray          *changed_medias)
+		 GrlSource    *source,
+		 GPtrArray    *changed_medias)
 {
 	GtkTreeModel *model;
 	guint i;
@@ -1084,8 +1134,8 @@ content_changed (TotemGrilo   *self,
 
 static void
 content_removed (TotemGrilo   *self,
-		 GrlSource          *source,
-		 GPtrArray          *changed_medias)
+		 GrlSource    *source,
+		 GPtrArray    *changed_medias)
 {
 	GtkTreeModel *model;
 	guint i;
@@ -1103,8 +1153,8 @@ content_removed (TotemGrilo   *self,
 
 static void
 content_added (TotemGrilo   *self,
-	       GrlSource          *source,
-	       GPtrArray          *changed_medias)
+	       GrlSource    *source,
+	       GPtrArray    *changed_medias)
 {
 	GtkTreeModel *model;
 	guint i;
@@ -1117,17 +1167,17 @@ content_added (TotemGrilo   *self,
 	for (i = 0; i < changed_medias->len; i++) {
 		GrlMedia *media = changed_medias->pdata[i];
 
-		add_local_metadata (self, media);
+		add_local_metadata (self, source, media);
 		add_media_to_model (GTK_TREE_STORE (model), NULL, source, media);
 	}
 }
 
 static void
-content_changed_cb (GrlSource          *source,
-		    GPtrArray          *changed_medias,
-		    GrlSourceChangeType change_type,
-		    gboolean            location_unknown,
-		    TotemGrilo   *self)
+content_changed_cb (GrlSource           *source,
+		    GPtrArray           *changed_medias,
+		    GrlSourceChangeType  change_type,
+		    gboolean             location_unknown,
+		    TotemGrilo          *self)
 {
 	switch (change_type) {
 	case GRL_CONTENT_CHANGED:
@@ -1148,8 +1198,8 @@ content_changed_cb (GrlSource          *source,
 
 static void
 source_added_cb (GrlRegistry *registry,
-                 GrlSource *source,
-                 gpointer user_data)
+                 GrlSource   *source,
+                 gpointer     user_data)
 {
 	const gchar *name;
 	TotemGrilo *self;
@@ -1184,13 +1234,14 @@ source_added_cb (GrlRegistry *registry,
 		self->priv->local_metadata_src = source;
 	else if (g_str_equal (id, "grl-metadata-store"))
 		self->priv->metadata_store_src = source;
+	else if (g_str_equal (id, "grl-bookmarks"))
+		self->priv->bookmarks_src = source;
 
 	ops = grl_source_supported_operations (source);
 	if (ops & GRL_OP_BROWSE) {
 		gboolean monitor = FALSE;
 
-		if (g_str_equal (id, "grl-tracker-source") ||
-		    g_str_equal (id, "grl-optical-media")) {
+		if (source_is_recent (source)) {
 			browse (self, self->priv->recent_model,
 				NULL, source, NULL, -1);
 			monitor = TRUE;
@@ -1211,9 +1262,12 @@ source_added_cb (GrlRegistry *registry,
 			monitor = TRUE;
 		}
 
-		if (monitor)
+		if (monitor &&
+		    (ops & GRL_OP_NOTIFY_CHANGE)) {
+			grl_source_notify_change_start (source, NULL);
 			g_signal_connect (G_OBJECT (source), "content-changed",
 					  G_CALLBACK (content_changed_cb), self);
+		}
 	}
 	if ((ops & GRL_OP_SEARCH) &&
 	    !source_is_search_blacklisted (source)) {
@@ -1226,9 +1280,9 @@ source_added_cb (GrlRegistry *registry,
 
 static gboolean
 remove_browse_result (GtkTreeModel *model,
-                      GtkTreePath *path,
-                      GtkTreeIter *iter,
-                      gpointer user_data)
+                      GtkTreePath  *path,
+                      GtkTreeIter  *iter,
+                      gpointer      user_data)
 {
 	GrlSource *removed_source = GRL_SOURCE (user_data);
 	GrlSource *model_source;
@@ -1250,8 +1304,8 @@ remove_browse_result (GtkTreeModel *model,
 
 static void
 source_removed_cb (GrlRegistry *registry,
-                   GrlSource *source,
-                   gpointer user_data)
+                   GrlSource   *source,
+                   gpointer     user_data)
 {
 	GrlSupportedOps ops;
 	TotemGrilo *self = TOTEM_GRILO (user_data);
@@ -1260,6 +1314,26 @@ source_removed_cb (GrlRegistry *registry,
 
 	/* Remove source and content from browse results */
 	if (ops & GRL_OP_BROWSE) {
+		/* Inside the removed browse source? */
+		if (self->priv->browser_filter_model) {
+			GtkTreePath *path;
+			GtkTreeIter iter;
+
+			g_object_get (G_OBJECT (self->priv->browser_filter_model), "virtual-root", &path, NULL);
+			if (path != NULL &&
+			    gtk_tree_model_get_iter (self->priv->browser_model, &iter, path)) {
+				GrlSource *current_source;
+
+				gtk_tree_model_get (self->priv->browser_model, &iter,
+						    MODEL_RESULTS_SOURCE, &current_source,
+						    -1);
+				if (current_source == source)
+					set_browser_filter_model_for_path (self, NULL);
+				g_clear_object (&current_source);
+			}
+			g_clear_pointer (&path, gtk_tree_path_free);
+		}
+
 		gtk_tree_model_foreach (self->priv->browser_model,
 		                        remove_browse_result,
 		                        source);
@@ -1317,23 +1391,6 @@ load_grilo_plugins (TotemGrilo *self)
 	}
 }
 
-static void
-unload_grilo_plugins (TotemGrilo *self)
-{
-	GrlRegistry *registry;
-	GList *l, *plugins;
-
-	registry = grl_registry_get_default ();
-	plugins = grl_registry_get_plugins (registry, TRUE);
-
-	for (l = plugins; l != NULL; l = l->next) {
-		GrlPlugin *plugin = l->data;
-		grl_registry_unload_plugin (registry, grl_plugin_get_id (plugin), NULL);
-	}
-
-	g_list_free (plugins);
-}
-
 static gboolean
 adjustment_over_limit (GtkAdjustment *adjustment)
 {
@@ -1360,7 +1417,7 @@ adjustment_changed_cb (GtkAdjustment *adjustment,
 
 static void
 get_more_browse_results_cb (GtkAdjustment *adjustment,
-                            TotemGrilo *self)
+                            TotemGrilo    *self)
 {
 	GtkTreeModel *model;
 	GtkIconView *icon_view;
@@ -1449,7 +1506,7 @@ get_more_browse_results_cb (GtkAdjustment *adjustment,
 
 static void
 adjustment_value_changed_cb (GtkAdjustment *adjustment,
-                             TotemGrilo *self)
+                             TotemGrilo    *self)
 {
 	update_search_thumbnails (self);
 
@@ -1515,20 +1572,31 @@ totem_grilo_back_button_clicked (TotemGrilo *self)
 }
 
 static gboolean
-window_key_press_event_cb (GtkWidget        *win,
-			   GdkEvent         *event,
-			   TotemGrilo *self)
+window_key_press_event_cb (GtkWidget   *win,
+			   GdkEventKey *event,
+			   TotemGrilo  *self)
 {
 	/* Check whether we're in the browse panel */
 	if (!g_str_equal (totem_object_get_main_page (self->priv->totem), "grilo"))
-		return FALSE;
+		return GDK_EVENT_PROPAGATE;
 
-	return gtk_search_bar_handle_event (GTK_SEARCH_BAR (self->priv->search_bar), event);
+	/* Handle Ctrl+F */
+	if (event->state != 0 &&
+	    (event->state & GDK_CONTROL_MASK)) {
+		if (event->keyval == GDK_KEY_F ||
+		    event->keyval == GDK_KEY_f) {
+			gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (self->priv->search_bar),
+							!gtk_search_bar_get_search_mode (GTK_SEARCH_BAR (self->priv->search_bar)));
+			return GDK_EVENT_STOP;
+		}
+	}
+
+	return gtk_search_bar_handle_event (GTK_SEARCH_BAR (self->priv->search_bar), (GdkEvent *) event);
 }
 
 static void
-selection_mode_requested (GdMainView       *view,
-			  TotemGrilo *self)
+selection_mode_requested (GdMainView  *view,
+			  TotemGrilo  *self)
 {
 	GtkTreePath *root = NULL;
 
@@ -1565,8 +1633,8 @@ can_remove_foreach (gpointer data,
 }
 
 static void
-view_selection_changed_cb (GdMainView       *view,
-			   TotemGrilo *self)
+view_selection_changed_cb (GdMainView   *view,
+			   TotemGrilo   *self)
 {
 	GtkTreeModel *model;
 	GList *list;
@@ -1594,7 +1662,7 @@ view_selection_changed_cb (GdMainView       *view,
 static void
 select_all_action_cb (GSimpleAction    *action,
 		      GVariant         *parameter,
-		      TotemGrilo *self)
+		      TotemGrilo       *self)
 {
 	gd_main_view_select_all (GD_MAIN_VIEW (self->priv->browser));
 }
@@ -1602,14 +1670,80 @@ select_all_action_cb (GSimpleAction    *action,
 static void
 select_none_action_cb (GSimpleAction    *action,
 		       GVariant         *parameter,
-		       TotemGrilo *self)
+		       TotemGrilo       *self)
 {
 	gd_main_view_unselect_all (GD_MAIN_VIEW (self->priv->browser));
 }
 
+static void
+totem_grilo_drop_files (TotemGrilo       *self,
+			GtkSelectionData *data)
+{
+	char **list;
+	guint i;
+
+	list = g_uri_list_extract_uris ((const char *) gtk_selection_data_get_data (data));
+
+	for (i = 0; list[i] != NULL; i++) {
+		if (list[i] == NULL)
+			continue;
+
+		g_debug ("Preparing to add '%s' as dropped file", list[i]);
+		totem_grilo_add_item_to_recent (self, list[i], NULL, FALSE);
+	}
+
+	g_strfreev (list);
+}
+
+static void
+drop_video_cb (GtkWidget          *widget,
+	       GdkDragContext     *context,
+	       gint                x,
+	       gint                y,
+	       GtkSelectionData   *data,
+	       guint               info,
+	       guint               _time,
+	       TotemGrilo         *self)
+{
+	GtkWidget *source_widget;
+	GdkDragAction action = gdk_drag_context_get_selected_action (context);
+
+	source_widget = gtk_drag_get_source_widget (context);
+
+	/* Drop of video on itself */
+	if (source_widget && widget == source_widget && action == GDK_ACTION_MOVE) {
+		gtk_drag_finish (context, FALSE, FALSE, _time);
+		return;
+	}
+
+	totem_grilo_drop_files (self, data);
+	gtk_drag_finish (context, TRUE, FALSE, _time);
+}
+
+static void
+totem_grilo_set_drop_enabled (TotemGrilo *self,
+			      gboolean    enabled)
+{
+	if (enabled == (self->priv->dnd_handler_id != 0))
+		return;
+
+	if (enabled) {
+		self->priv->dnd_handler_id = g_signal_connect (G_OBJECT (self->priv->browser), "drag_data_received",
+							       G_CALLBACK (drop_video_cb), self);
+		gtk_drag_dest_set (GTK_WIDGET (self->priv->browser), GTK_DEST_DEFAULT_ALL,
+				   target_table, G_N_ELEMENTS (target_table),
+				   GDK_ACTION_MOVE | GDK_ACTION_COPY);
+	} else {
+		g_signal_handler_disconnect (G_OBJECT (self->priv->browser),
+					     self->priv->dnd_handler_id);
+		self->priv->dnd_handler_id = 0;
+		gtk_drag_dest_unset (GTK_WIDGET (self->priv->browser));
+	}
+}
+
 static void
 source_switched (GtkToggleButton  *button,
-		 TotemGrilo *self)
+		 TotemGrilo       *self)
 {
 	const char *id;
 
@@ -1620,15 +1754,24 @@ source_switched (GtkToggleButton  *button,
 	if (g_str_equal (id, "recent")) {
 		gd_main_view_set_model (GD_MAIN_VIEW (self->priv->browser),
 					self->priv->recent_sort_model);
+		self->priv->current_page = TOTEM_GRILO_PAGE_RECENT;
+		totem_grilo_set_drop_enabled (self, TRUE);
 	} else if (g_str_equal (id, "channels")) {
 		if (self->priv->browser_filter_model != NULL)
 			gd_main_view_set_model (GD_MAIN_VIEW (self->priv->browser),
 						self->priv->browser_filter_model);
 		else
 			set_browser_filter_model_for_path (self, NULL);
-	} else {
-		g_assert_not_reached ();
+		self->priv->current_page = TOTEM_GRILO_PAGE_CHANNELS;
+		totem_grilo_set_drop_enabled (self, FALSE);
+	} else if (g_str_equal (id, "search")) {
+		return;
 	}
+
+	g_clear_pointer (&self->priv->last_page, g_free);
+	g_object_set (self->priv->header, "search-mode", FALSE, NULL);
+
+	g_object_notify (G_OBJECT (self), "current-page");
 }
 
 static GtkWidget *
@@ -1661,11 +1804,22 @@ setup_source_switcher (TotemGrilo *self)
 	self->priv->recent = create_switcher_button (self, _("Recent"), "recent");
 	gtk_container_add (GTK_CONTAINER (self->priv->switcher), self->priv->recent);
 
+	g_signal_connect (G_OBJECT (self->priv->recent), "drag_data_received",
+			  G_CALLBACK (drop_video_cb), self);
+	gtk_drag_dest_set (GTK_WIDGET (self->priv->recent), GTK_DEST_DEFAULT_ALL,
+			   target_table, G_N_ELEMENTS (target_table),
+			   GDK_ACTION_MOVE | GDK_ACTION_COPY);
+
 	self->priv->channels = create_switcher_button (self, _("Channels"), "channels");
 	gtk_radio_button_join_group (GTK_RADIO_BUTTON (self->priv->channels),
 				     GTK_RADIO_BUTTON (self->priv->recent));
 	gtk_container_add (GTK_CONTAINER (self->priv->switcher), self->priv->channels);
 
+	self->priv->search_hidden_button = create_switcher_button (self, "HIDDEN SEARCH BUTTON", "search");
+	gtk_radio_button_join_group (GTK_RADIO_BUTTON (self->priv->search_hidden_button),
+				     GTK_RADIO_BUTTON (self->priv->recent));
+	g_object_ref_sink (G_OBJECT (self->priv->search_hidden_button));
+
 	context = gtk_widget_get_style_context (self->priv->switcher);
 	gtk_style_context_add_class (context, "stack-switcher");
 	gtk_style_context_add_class (context, GTK_STYLE_CLASS_LINKED);
@@ -1677,16 +1831,22 @@ setup_source_switcher (TotemGrilo *self)
 static void
 search_mode_changed (GObject          *gobject,
 		     GParamSpec       *pspec,
-		     TotemGrilo *self)
+		     TotemGrilo       *self)
 {
 	gboolean search_mode;
 
 	search_mode = totem_main_toolbar_get_search_mode (TOTEM_MAIN_TOOLBAR (self->priv->header));
 	if (!search_mode) {
-		/* One of those will fail, as there's the toggle
-		 * button won't be active */
-		source_switched (GTK_TOGGLE_BUTTON (self->priv->recent), self);
-		source_switched (GTK_TOGGLE_BUTTON (self->priv->channels), self);
+		if (self->priv->last_page ==  NULL) {
+			/* Already reset */
+		} else if (g_str_equal (self->priv->last_page, "recent")) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->recent), TRUE);
+		} else if (g_str_equal (self->priv->last_page, "channels")) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->channels), TRUE);
+		} else {
+			g_assert_not_reached ();
+		}
+		g_clear_pointer (&self->priv->last_page, g_free);
 	} else {
 		GtkTreeModel *model;
 		const char *id = NULL;
@@ -1695,6 +1855,7 @@ search_mode_changed (GObject          *gobject,
 		model = gd_main_view_get_model (GD_MAIN_VIEW (self->priv->browser));
 		if (model == self->priv->recent_sort_model) {
 			id = "grl-tracker-source";
+			self->priv->last_page = g_strdup ("recent");
 		} else {
 			GtkTreeIter iter;
 			GtkTreePath *path;
@@ -1710,10 +1871,15 @@ search_mode_changed (GObject          *gobject,
 				id = source ? grl_source_get_id (source) : NULL;
 				g_clear_object (&source);
 			}
+			g_clear_pointer (&path, gtk_tree_path_free);
+
+			self->priv->last_page = g_strdup ("channels");
 		}
 
 		if (id != NULL)
 			totem_search_entry_set_selected_id (TOTEM_SEARCH_ENTRY (self->priv->search_entry), id);
+
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->search_hidden_button), TRUE);
 	}
 
 	self->priv->in_search = search_mode;
@@ -1770,7 +1936,7 @@ shuffle_items (GList *list)
 
 static void
 play_selection (TotemGrilo *self,
-		gboolean          shuffle)
+		gboolean    shuffle)
 {
 	GtkTreeModel *model;
 	GList *list;
@@ -1834,14 +2000,14 @@ next_item:
 
 static void
 play_cb (TotemSelectionToolbar *bar,
-	 TotemGrilo      *self)
+	 TotemGrilo            *self)
 {
 	play_selection (self, FALSE);
 }
 
 static void
 shuffle_cb (TotemSelectionToolbar *bar,
-	    TotemGrilo      *self)
+	    TotemGrilo            *self)
 {
 	play_selection (self, TRUE);
 }
@@ -1866,18 +2032,29 @@ delete_foreach (gpointer data,
 
 	success = TRUE;
 	if (grl_source_supported_operations (source) & GRL_OP_REMOVE) {
+		g_debug ("Removing item '%s' through Grilo",
+			 grl_media_get_id (media));
 		grl_source_remove_sync (source, media, &error);
 		success = (error != NULL);
 	} else {
-		GFile *file;
+		const char *uri;
 
-		file = g_file_new_for_uri (grl_media_get_url (media));
-		success = g_file_trash (file, NULL, &error);
-		g_object_unref (file);
+		uri = grl_media_get_url (media);
+		if (!uri) {
+			g_set_error_literal (&error, G_IO_ERROR, G_IO_ERROR_NOT_SUPPORTED,
+					     "Item cannot be removed through Grilo and doesn't have a URI, please file a bug");
+		} else {
+			GFile *file;
+
+			file = g_file_new_for_uri (grl_media_get_url (media));
+			success = g_file_trash (file, NULL, &error);
+			g_object_unref (file);
+		}
 	}
 	if (!success) {
-		g_warning ("Couldn't remove item '%s': %s",
+		g_warning ("Couldn't remove item '%s' (%s): %s",
 			   grl_media_get_title (media),
+			   grl_media_get_id (media),
 			   error->message);
 		g_error_free (error);
 	} else {
@@ -1905,7 +2082,7 @@ delete_foreach (gpointer data,
 
 static void
 delete_cb (TotemSelectionToolbar *bar,
-	   TotemGrilo      *self)
+	   TotemGrilo            *self)
 {
 	GtkTreeModel *model;
 	GList *list;
@@ -1985,6 +2162,8 @@ setup_browse (TotemGrilo *self)
 	g_signal_connect (self->priv->browser, "selection-mode-request",
 			  G_CALLBACK (selection_mode_requested), self);
 
+	totem_grilo_set_drop_enabled (self, TRUE);
+
 	/* Selection toolbar */
 	g_object_set (G_OBJECT (self->priv->header), "select-menu-model", self->priv->selectmenu, NULL);
 	self->priv->selection_bar = totem_selection_toolbar_new ();
@@ -2070,7 +2249,7 @@ remaining_to_text (GtkTreeViewColumn *column,
 }
 
 static void
-create_debug_window (TotemGrilo *self,
+create_debug_window (TotemGrilo       *self,
 		     GtkTreeModel     *model)
 {
 	GtkWidget *window, *scrolled, *tree;
@@ -2171,12 +2350,12 @@ totem_grilo_finalize (GObject *object)
 
 	g_clear_pointer (&self->priv->metadata_keys, g_list_free);
 
-	/* Shutdown all plugins */
-	unload_grilo_plugins (self);
+	grl_deinit ();
 
 	totem_grilo_clear_icons ();
 
 	g_clear_object (&self->priv->switcher);
+	g_clear_object (&self->priv->search_hidden_button);
 
 	/* Empty results */
 	//FIXME
@@ -2212,6 +2391,94 @@ totem_grilo_get_show_back_button (TotemGrilo *self)
 	return self->priv->show_back_button;
 }
 
+void
+totem_grilo_set_current_page (TotemGrilo     *self,
+			      TotemGriloPage  page)
+{
+	GtkWidget *button;
+
+	g_return_if_fail (TOTEM_IS_GRILO (self));
+
+	if (page == TOTEM_GRILO_PAGE_RECENT)
+		button = self->priv->recent;
+	else if (page == TOTEM_GRILO_PAGE_CHANNELS)
+		button = self->priv->channels;
+	else
+		g_assert_not_reached ();
+
+	self->priv->current_page = page;
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
+
+	g_object_notify (G_OBJECT (self), "current-page");
+}
+
+TotemGriloPage
+totem_grilo_get_current_page (TotemGrilo *self)
+{
+	g_return_val_if_fail (TOTEM_IS_GRILO (self), TOTEM_GRILO_PAGE_RECENT);
+
+	return self->priv->current_page;
+}
+
+gboolean
+totem_grilo_add_item_to_recent (TotemGrilo *self,
+				const char *uri,
+				const char *title,
+				gboolean    is_web)
+{
+	GrlMedia *media;
+	GFile *file;
+
+	g_return_val_if_fail (TOTEM_IS_GRILO (self), FALSE);
+
+	file = g_file_new_for_uri (uri);
+
+	if (is_web) {
+		char *basename;
+
+		/* This should only come from the
+		 * "Open Location" dialogue, so it
+		 * shouldn't have a title */
+		g_assert (title == NULL);
+
+		media = grl_media_video_new ();
+
+		basename = g_file_get_basename (file);
+		grl_media_set_title (media, basename);
+		g_free (basename);
+
+		grl_media_set_url (media, uri);
+	} else {
+		GrlOperationOptions *options;
+
+		options = grl_operation_options_new (NULL);
+		media = grl_media_video_new ();
+		media = grl_pls_file_to_media (media,
+					       file,
+					       NULL,
+					       FALSE,
+					       options);
+		if (media && title)
+			grl_media_set_title (media, title);
+
+		g_object_unref (options);
+	}
+
+	g_object_unref (file);
+
+	if (!media)
+		return FALSE;
+
+	/* This should be quick, just adding the item to the DB */
+	grl_source_store_sync (self->priv->bookmarks_src,
+			       NULL,
+			       media,
+			       GRL_WRITE_NORMAL,
+			       NULL);
+	return TRUE;
+}
+
 static void
 totem_grilo_set_property (GObject         *object,
                           guint            prop_id,
@@ -2236,6 +2503,10 @@ totem_grilo_set_property (GObject         *object,
 		g_object_set (self->priv->header, "show-back-button", priv->show_back_button, NULL);
 		break;
 
+	case PROP_CURRENT_PAGE:
+		totem_grilo_set_current_page (self, g_value_get_int (value));
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -2255,6 +2526,10 @@ totem_grilo_get_property (GObject         *object,
 		g_value_set_boolean (value, self->priv->show_back_button);
 		break;
 
+	case PROP_CURRENT_PAGE:
+		g_value_set_int (value, self->priv->current_page);
+		break;
+
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -2296,6 +2571,14 @@ totem_grilo_class_init (TotemGriloClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
+	g_object_class_install_property (object_class,
+					 PROP_CURRENT_PAGE,
+					 g_param_spec_int ("current-page",
+							   "Current page",
+							   "The name of the currently visible page",
+							   TOTEM_GRILO_PAGE_RECENT, TOTEM_GRILO_PAGE_CHANNELS, TOTEM_GRILO_PAGE_RECENT,
+							   G_PARAM_READWRITE));
+
 	gtk_widget_class_set_template_from_resource (widget_class, "/org/totem/grilo/grilo.ui");
 	gtk_widget_class_bind_template_child_private (widget_class, TotemGrilo, selectmenu);
 	gtk_widget_class_bind_template_child_private (widget_class, TotemGrilo, search_bar);
