@@ -349,29 +349,25 @@ totem_interface_set_transient_for (GtkWindow *window, GtkWindow *parent)
 				      GTK_WINDOW (parent));
 }
 
-char *
-totem_interface_get_license (void)
+GtkWidget *
+totem_interface_create_header_button (GtkWidget  *header,
+				      GtkWidget  *button,
+				      const char *icon_name,
+				      GtkPackType pack_type)
 {
-	const char *license[] = {
-		N_("Totem is free software; you can redistribute it and/or modify "
-		   "it under the terms of the GNU General Public License as published by "
-		   "the Free Software Foundation; either version 2 of the License, or "
-		   "(at your option) any later version."),
-		N_("Totem is distributed in the hope that it will be useful, "
-		   "but WITHOUT ANY WARRANTY; without even the implied warranty of "
-		   "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
-		   "GNU General Public License for more details."),
-		N_("You should have received a copy of the GNU General Public License "
-		   "along with Totem; if not, write to the Free Software Foundation, Inc., "
-		   "59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"),
-		N_("Totem contains an exception to allow the use of proprietary "
-		   "GStreamer plugins.")
-	};
-	return g_strjoin ("\n\n",
-			  _(license[0]),
-			  _(license[1]),
-			  _(license[2]),
-			  _(license[3]),
-			  NULL);
-}
+	GtkWidget *image;
+	GtkStyleContext *context;
 
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	gtk_button_set_image (GTK_BUTTON (button), image);
+	context = gtk_widget_get_style_context (button);
+	gtk_style_context_add_class (context, "image-button");
+	g_object_set (G_OBJECT (button), "valign", GTK_ALIGN_CENTER, NULL);
+
+	if (pack_type == GTK_PACK_END)
+		gtk_header_bar_pack_end (GTK_HEADER_BAR (header), button);
+	else
+		gtk_header_bar_pack_start (GTK_HEADER_BAR (header), button);
+
+	return button;
+}
