@@ -147,16 +147,20 @@ update_toolbar_state (TotemMainToolbar *bar)
     }
   else if (priv->search_mode)
     {
-      char *label;
-
-      gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), SEARCH_RESULTS_PAGE);
       if (!priv->search_string || *priv->search_string == '\0')
-        label = g_strdup ("");
+        {
+          gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), CUSTOM_TITLE_PAGE);
+        }
       else
-        label = g_strdup_printf (_("Results for “%s”"), priv->search_string);
+        {
+          char *label;
 
-      gtk_label_set_label (GTK_LABEL (priv->search_results_label), label);
-      g_free (label);
+          gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), SEARCH_RESULTS_PAGE);
+          label = g_strdup_printf (_("Results for “%s”"), priv->search_string);
+
+          gtk_label_set_label (GTK_LABEL (priv->search_results_label), label);
+          g_free (label);
+        }
 
       if (priv->show_select_button)
         gtk_widget_show (priv->select_button);
@@ -439,14 +443,6 @@ totem_main_toolbar_class_init (TotemMainToolbarClass *klass)
                 g_cclosure_marshal_generic,
                 G_TYPE_NONE, 0, G_TYPE_NONE);
 
-  g_signal_new ("new-clicked",
-                G_OBJECT_CLASS_TYPE (klass),
-                0,
-                0,
-                NULL, NULL,
-                g_cclosure_marshal_generic,
-                G_TYPE_NONE, 0, G_TYPE_NONE);
-
   gtk_widget_class_set_template_from_resource (widget_class, "/org/totem/grilo/totemmaintoolbar.ui");
   gtk_widget_class_bind_template_child_private (widget_class, TotemMainToolbar, search_button);
   gtk_widget_class_bind_template_child_private (widget_class, TotemMainToolbar, select_button);
